@@ -1,5 +1,8 @@
 context("code strings")
 
+test_that("as_monad captures code string", {
+  expect_equal(get_code(as_monad(41+1))[[1]], '41 + 1') 
+})
 
 test_that("correct for %>>%", {
   expect_equal(
