@@ -12,24 +12,17 @@ test_that("missues finds the correct errors and warnings", {
   expect_silent(234 %>>% sqrt %>% esc)
   expect_warning(warning("foo") %__% 5 %>% esc, "foo") 
   expect_equal(
-    lapply(
-      funnel({warning("goo"); warning("roo")} %__% warning("poo") %>% esc(quiet=TRUE)),
-      m_warnings
-    )[[1]],
-    c("goo", "roo", "poo")
+    funnel(
+      {warning("goo"); warning("roo")} %__% warning("poo") %>% esc(quiet=TRUE)
+    ) %>% get_warnings,
+    list(c("goo", "roo", "poo"), character(0))
   )
   expect_equal(
-    lapply(
-      funnel(warning("goo") %__% warning("roo") %>% esc(quiet=TRUE)),
-      m_warnings
-    )[[1]],
-    c("goo", "roo")
+    funnel(warning("goo") %__% warning("roo") %>% esc(quiet=TRUE)) %>% get_warnings,
+    list(c("goo", "roo"), character(0))
   )
   expect_equal(
-    lapply(
-      funnel(-2:3 %>>% sqrt %>>% sum %>% esc(quiet=TRUE)),
-      m_warnings
-    )[[1]],
-    "NaNs produced"
+    funnel(-2:3 %>>% sqrt %>>% sum %>% esc(quiet=TRUE)) %>% get_warnings,
+    list("NaNs produced", character(0))
   )
 })
